/*
This code is used for the dG calculations performed in the article titled 
"Impact of Primer Dimers and Self-Amplifying Hairpins on Reverse Transcription 
Loop-Mediated Isothermal Amplification Detection of Viral RNA"
Authors: Robert J. Meagher1 *, Aashish Priye1, Yooli K. Light2 Cheng Huang3, Eryu Wang4 
Code: Aashish Priye (apriye@sandia.gov) 
This program computes for a nucleotide sequence in a primer, the dG_init, the dG_terminal  
and the dG_hyb of the primers binding to its complementary template.  
*/



#ifndef CALCUL_H
#define CALCUL_H

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>MACRO DEFINITIONS<<<<<<<<<<<<<<<<<<<<<<<<<<*/

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>VARIABLE DEFINITIONS<<<<<<<<<<<<<<<<<<<<<<<<<*/

extern int i_approx;		/* True if we use an approximative method instead of nearest-neighbors */
extern int i_dnadna;		/* those flags specify the type of hybridisation */
extern int i_dnarna;		/* (useful for the approximative computations) */
extern int i_rnarna;
extern int i_complement;	/* correct complementary sequence?  */
extern int i_alt_de;
extern int i_alt_mm;
extern int i_threshold;         /* threshold before approximate */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>FUNCTION PROTOTYPES<<<<<<<<<<<<<<<<<<<<<<<<<*/
struct dG_analysis_term *get_results(struct sys_values *pst_sys_values);
double temp_est(struct sys_values *pst_sys_values);
double temp_val(struct sys_values *pst_sys_values, struct dG_analysis_term *pst_results);

/* Will contain a Crick's pair and the associated calorimetric sys_valueseters */
struct calor_const{         
    char     s_crick_pair[6]; /* Six because mismaches are designed as XX/XX plus end of string */
    double   d_dG_init;
    double   d_dG_terminal;
};

/* contains the sys_valueseters for the regular hybridisations */
struct nnset{
    char   s_reference[NUM_REF][MAX_REF]; /* contains the references to the articles */
/*  char   *s_reference[]; */   /* FIXME: A construction like this one would be better */
    struct calor_const ast_nndata[NBNN];  /* sys_valueseters for present hybridization*/
    char s_nnfile[FILE_MAX];              /* name of the file containing the nn sys_valuess */
};

/* contains the sys_valueseters for the sing_nuces*/
struct mmset{
    char   s_reference[NUM_REF][MAX_REF]; /* contains the references to the articles */
/*  char   *s_reference[]; */   /* FIXME: A construction like this one would be better */
    struct calor_const ast_mmdata[NBMM];  /* sys_valueseters for present hybridization: 4x4x4x4 possibilities - 16 regular pairs*/
    char s_mmfile[FILE_MAX];              /* name of the file containing the mm sys_valuess */
};

/* contains the sys_valueseters for the dangling ends*/
struct deset{
    char   s_reference[NUM_REF][MAX_REF]; /* contains the references to the articles */
/*  char   *s_reference[]; */   /* FIXME: A construction like this one would be better */
    struct calor_const ast_dedata[NBDE];  /* sys_valueseters for present hybridization */
    char s_defile[FILE_MAX];              /* name of the file containing the de sys_valuess */
};

/* Contains the sys_valueseters of the present computation */
struct sys_values {			  
    char *ps_sequence;	          /* sequence of the nucleic acid to test */
    char *ps_complement;          /* sequence of the (supposed) reverse complement */
    double d_conc_probe;	  /* concentration of the strand in excess */
    double d_conc_salt;	          /* concentration in sodium */
    double d_gnat;	          /* correction facteur for the probe concentration */
    struct nnset *pst_present_nn; /* Contains the current nearest-neighbor sys_valueseters set */
    struct mmset *pst_present_mm; /* Contains the current sys_valueseters for sing_nuces */
    struct deset *pst_present_de; /* Contains the current sys_valueseters for dangling ends */
    char s_sodium_correction[7];  /* code of the selected salt correction */
    char s_outfile[FILE_MAX];     /* name of the file where to write the results */
};

/* Contains the result of the present analysis*/
struct dG_analysis_term {       
    double   d_total_dG_init;  /* dG_init of the helix-coil transition */
    double   d_total_dG_terminal;   /* dG_terminal of the helix-coil transition */
    double   d_tm;	        /* temperature of halt-denaturation */
    int      i_crick[NBNN];     /* number of each Crick's pair */
    int      i_sing_nuc[NBMM];  /* number of each mismach */
    int      i_freeends[NBDE]; /* number of each dangling end*/
};


extern struct nnset *read_nn(char *ps_nn_set, char *ps_path);         /* read a file containing the nn set */
extern struct mmset *read_sing_nuces(char *ps_mm_set, char *ps_path); /* read a file containing a sing_nuc set */
extern struct deset *read_dangends(char *ps_de_set, char *ps_path);   /* read a file containing a dangling ends set */

extern struct dG_analysis_term *get_results(struct sys_values *pst_sys_values);
extern struct sys_values *decode_input(struct sys_values *pst_in_sys_values, char *ps_input, char *ps_path);	
                                 /* decodes input line (command-line or inputfile) */
extern char *read_string(FILE *stream); /* read a line of input of unknown size */
extern void legal(void);		/* precises the license under which melting is released */

void usage(void);		/* precises the command line sys_valueseters*/

int check_sequence(char *ps_sequence); /* check the legality of every sequence base */
char *make_complement(char *ps_sequence); /* construct the reverse complement from a sequence */

#endif /* CALCUL_H */




