% This code is used for the dG calculations performed in the article titled
% "Impact of Primer Dimers and Self-Amplifying Hairpins on Reverse Transcription
% Loop-Mediated Isothermal Amplification Detection of Viral RNA"
% Authors: Robert J. Meagher1 *, Aashish Priye1, Yooli K. Light2 Cheng Huang3, Eryu Wang4
% Code: Aashish Priye (apriye@sandia.gov)
% This program computes for a nucleotide sequence in a primer, the dG_init, the dG_terminal
% and the dG_hyb of the primers binding to its complementary template.


% Nearest Neighbour stability matrix
% 
% AA  -7900.0  -22.2
% AC  -8400.0  -22.4
% AG  -7800.0  -21.0
% AT  -7200.0  -20.4
% CA  -8500.0  -22.7
% CC  -8000.0  -19.9
% CG -10600.0  -27.2
% CT  -7800.0  -21.0
% GA  -8200.0  -22.2
% GC  -9800.0  -24.4
% GG  -8000.0  -19.9
% GT  -8400.0  -22.4
% TA  -7200.0  -21.3
% TC  -8200.0  -22.2 
% TG  -8500.0  -22.7
% TT  -7900.0  -22.2
% IA   2300.0    4.1
% IG    100.0   -2.8
% 
% REF: Allawi and SantaLucia (1997). Biochemistry 36 : 10581-10594

clear all;
clc;
nn_stabilit_matrix = [-7900.0,  -22.2;  -8400.0,  -22.4;  -7800.0,  -21.0;  -7200.0,  -20.4;  -8500.0,  -22.7;  -8000.0,  -19.9; -10600.0,  -27.2;  -7800.0,  -21.0;  -8200.0,  -22.2;  -9800.0,  -24.4;  -8000.0,  -19.9;  -8400.0,  -22.4;  -7200.0,  -21.3;  -8200.0,  -22.2;  -8500.0,  -22.;-7900.0,  -22.2;2300.0,    4.1;    100.0,   -2.8]; % Stability matrix of the Nearest neighbor pair 
%i,j;			% base pair counters %
%i_sing_nuc;             % sing_nuce detector %
%i_freeend;              % free end of nucleotide%
i_length = 0;		% length of the sequence %
i_proxoffset = 0;       % current position from the sequence end%
i_distoffset = 0;       % current distance from the sequence end%
struct dG_analysis_term *pst_results; % contains the results ... %
[num,primer_data,raw] = xlsread('filename.xlsx'); % The excel sheet should contain all the primer sequences of one set with the first primer sequence being the one with the modification
base_seq_primer1 = primer_data(1);    % extract primer sequecnce
base_seq_primer2 = primer_data(2);    % extract primer sequecnce
base_seq_primer3 = primer_data(3);    % extract primer sequecnce
base_seq_primer4 = primer_data(4);    % extract primer sequecnce
base_seq_primer5 = primer_data(5);    % extract primer sequecnce
base_seq_primer6 = primer_data(6);    % extract primer sequecnce
%Strand(:,1,:) = Strand number
%Strand(:,2,:) = Nucleotide segment
%Strand(:,3,:) = Covalend bonded number
%Strand(:,4,:) = Five prime bonded number
%Strand(:,5,:) = Three prime bonded number

THREE_PRIME=5;
FIVE_PRIME=4;
COVE_BOND=3;
NUC_SEG=2;
STRAND_NUM=1;

%Notation for sequence search and annealing
Strand(:,STRAND_NUM,1)=[1,2,3,4,5,6,7,8,9];
Strand(:,2,1)=[1,2,3,4,5,6,7,8,9];
Strand(:,3,1)=[0,0,0,0,0,0,0,0,0];
Strand(:,4,1)=[2,3,4,5,6,7,8,9,0];
Strand(:,5,1)=[0,1,2,3,4,5,6,7,8];
BIP(:,STRAND_NUM)=[1,2];BIP(:,2)=[3,-4];BIP(:,3)=[0,0];BIP(:,4)=[2,0];BIP(:,5)=[0,1];
FIP(:,STRAND_NUM)=[1,2];FIP(:,2)=[-7,6];FIP(:,3)=[0,0];FIP(:,4)=[0,1];FIP(:,5)=[2,0];
B3(:,STRAND_NUM)=[1];B3(:,2)=[2];B3(:,3)=[0];B3(:,4)=[0];B3(:,5)=[0];
F3(:,STRAND_NUM)=[1];F3(:,2)=[-8];F3(:,3)=[0];F3(:,4)=[0];F3(:,5)=[0];

strand_count=1;

[t1,t2,size_struct]=size(Strand(1,STRAND_NUM,:));
current_strand=1;

[num,c_file_output_results,raw] = xlsread('get_results.xlsx'); %import_data from c file
[num,c_file_output_parameters,raw] = xlsread('pst_sys_values.xlsx'); %import_data from c file



%% Analyze primer sequences and export the base pairs to be analyzed by dG analysis file
while (current_strand<=size_struct) %translate through each strand in the strand matrix one by one
    
    %[size_strand,t1]=size(Strand(:,1,current_strand));
    %working_strand=current_strand; length_strand;
    
    lastnode_strand=current_strand;
    working_strandsize=lastnode;
    
    
    count_BIP1=0;count_BIP2=0;count_FIP1=0;count_FIP2=0;count_F3=0;count_B3=0;
    count_self=0;
    
    for strand_seg_no=1:working_strandsize
        if ( (Strand(strand_seg_no,NUC_SEG,current_strand)==-BIP(1,2)) && (Strand(strand_seg_no,COVE_BOND,current_strand)==0) )
            BIP1_site(count_BIP1+1)=Strand(strand_seg_no,STRAND_NUM,current_strand);
            count_BIP1=count_BIP1+1;
            export_dG_analysis(Strand(strand_seg_no,NUC_SEG,current_strand),count);
        end
        
        if ( (Strand(strand_seg_no,NUC_SEG,current_strand)==-BIP(2,2)) && (Strand(strand_seg_no,COVE_BOND,current_strand)==0) )
            BIP2_site(count_BIP2+1)=Strand(strand_seg_no,STRAND_NUM,current_strand);
            count_BIP2=count_BIP2+1;
            export_dG_analysis(Strand(strand_seg_no,NUC_SEG,current_strand),count);
        end
        
        if ( (Strand(strand_seg_no,NUC_SEG,current_strand)==-FIP(1,2)) && (Strand(strand_seg_no,COVE_BOND,current_strand)==0) )
            FIP1_site(count_FIP1+1)=Strand(strand_seg_no,STRAND_NUM,current_strand,count);
            count_FIP1=count_FIP1+1;
            export_dG_analysis(Strand(strand_seg_no,NUC_SEG,current_strand));
        end
        
        if ( (Strand(strand_seg_no,NUC_SEG,current_strand)==-FIP(2,2)) && (Strand(strand_seg_no,COVE_BOND,current_strand)==0) )
            FIP2_site(count_FIP2+1)=Strand(strand_seg_no,STRAND_NUM,current_strand);
            count_FIP2=count_FIP2+1;
            export_dG_analysis(Strand(strand_seg_no,NUC_SEG,current_strand),count);
        end
        
        if ( (Strand(strand_seg_no,NUC_SEG,current_strand)==-F3(1,2)) && (Strand(strand_seg_no,COVE_BOND,current_strand)==0) )
            F3_site(count_F3+1)=Strand(strand_seg_no,STRAND_NUM,current_strand);
            count_F3=count_F3+1;
        end
        
        if ( (Strand(strand_seg_no,NUC_SEG,current_strand)==-B3(1,2)) && (Strand(strand_seg_no,COVE_BOND,current_strand)==0) )
            B3_site(count_B3+1)=Strand(strand_seg_no,STRAND_NUM,current_strand);
            count_B3=count_B3+1;
        end
        
        for j=1:working_strandsize
            if ( (Strand(strand_seg_no,NUC_SEG,current_strand)==-(Strand(j,NUC_SEG,current_strand)) && (Strand(strand_seg_no,COVE_BOND,current_strand)==0) && (Strand(j,COVE_BOND,current_strand)==0))  )
                self_site(count_self+1,1)=Strand(strand_seg_no,STRAND_NUM,current_strand);  %self priming location 1
                self_site(count_self+1,2)=Strand(j,STRAND_NUM,current_strand);               %self priming location 2
                count_self=count_self+1;
                export_dG_analysis(Strand(strand_seg_no,NUC_SEG,current_strand),count);
            end
        end
    end
    
    if (count_F3>0)
        [temp_size,t1]=size(F3_site);
        for i=1:temp_size
            %          strand_count=strand_count+1; % Inrease the strand count in the strand matrix
            attach_site=F3_site(i);  %the node number where the primer attaches
            %         new_strand=new_strand+1;    %new strand
            add_strand;
            %         Strand(:,:,new_strand)=Strand(:,:,current_strand);
            Primer_anneal;
            Primer_attach_extend;       %run primer attach and extend routine
            export_dG_analysis(Strand(strand_seg_no,NUC_SEG,current_strand),count);
        end
    end
    
    if (count_B3>0)
        [temp_size,t1]=size(B3_site);
        for i=1:temp_size
            %          strand_count=strand_count+1; % Inrease the strand count in the strand matrix
            attach_site=B3_site(i);  %the node number where the primer attaches
            %          new_strand=new_strand+1;    %new strand
            add_strand;
            Primer_anneal;
            Primer_attach_extend;       %run primer attach and extend routine
            export_dG_analysis(Strand(strand_seg_no,NUC_SEG,current_strand),count);
        end
    end
    
    if (count_FIP1>0)
        [temp_size,t1]=size(FIP1_site);
        for i=1:temp_size
            %           strand_count=strand_count+1; % Inrease the strand count in the strand matrix
            attach_site=FIP1_site(i);  %the node number where the primer attaches
            %          new_strand=new_strand+1;    %new strand
            add_strand;
            %add other nuc segment of the primer
            Primer_anneal;
            Identify_lastnode;
            Strand(lastnode+1,STRAND_NUM,new_strand) = 1 + Strand(lastnode,STRAND_NUM,new_strand);  %Add a node at the end for the new nucleotide segment
            Strand(lastnode+1,NUC_SEG,new_strand)= FIP(2,2);  %Write the nucleotide segment on the new node
            Strand(lastnode+1,COVE_BOND,new_strand)=0;
            Strand(lastnode+1,FIVE_PRIME,new_strand)=Strand(Strand(attach_site,COVE_BOND,new_strand),STRAND_NUM,new_strand);
            Strand(lastnode+1,THREE_PRIME,new_strand)=0;
            Strand( look_for_node(    (Strand(lastnode+1,FIVE_PRIME,new_strand)),   Strand(:,:,new_strand)  ),THREE_PRIME,new_strand) = Strand(lastnode+1,STRAND_NUM,new_strand);
            Primer_attach_extend;       %run primer attach and extend routine
            
        end
        
    end
    
    if (count_FIP2>0)
        [temp_size,t1]=size(FIP2_site);
        for i=1:temp_size
            %        strand_count=strand_count+1; % Inrease the strand count in the strand matrix
            attach_site=FIP2_site(i);  %the node number where the primer attaches
            %           new_strand=new_strand+1;    %new strand
            add_strand;
            Primer_anneal;
            
            %add other nuc segment of the primer
            Identify_lastnode;
            Strand(lastnode+1,STRAND_NUM,new_strand) = 1 + Strand(lastnode,STRAND_NUM,new_strand);  %Add a node at the end for the new nucleotide segment
            Strand(lastnode+1,NUC_SEG,new_strand)= FIP(1,2);  %Write the nucleotide segment on the new node
            Strand(lastnode+1,COVE_BOND,new_strand)=0;
            Strand(lastnode+1,FIVE_PRIME,new_strand)=0;
            Strand(lastnode+1,THREE_PRIME,new_strand)=Strand(Strand(attach_site,COVE_BOND,new_strand),STRAND_NUM,new_strand);
            Primer_attach_extend;       %run primer attach and extend routine
        end
    end
    
    if (count_BIP1>0)
        [temp_size,t1]=size(BIP1_site);
        for i=1:temp_size
            %         strand_count=strand_count+1; % Inrease the strand count in the strand matrix
            attach_site=BIP1_site(i);  %the node number where the primer attaches
            %           new_strand=new_strand+1;    %new strand
            add_strand;
            
            
            %add other nuc segment of the primer
            
            Primer_anneal;
            Identify_lastnode;
            Strand(lastnode+1,STRAND_NUM,new_strand) = 1 + Strand(lastnode,STRAND_NUM,new_strand);  %Add a node at the end for the new nucleotide segment
            Strand(lastnode+1,NUC_SEG,new_strand)= FIP(2,2);  %Write the nucleotide segment on the new node
            Strand(lastnode+1,COVE_BOND,new_strand)=0;
            Strand(lastnode+1,FIVE_PRIME,new_strand)=Strand(Strand(attach_site,COVE_BOND,new_strand),STRAND_NUM,new_strand);
            Strand(lastnode+1,THREE_PRIME,new_strand)=0;
            Primer_attach_extend;       %run primer attach and extend routine
        end
    end
    
    if (count_BIP2>0)
        [temp_size,t1]=size(BIP2_site);
        for i=1:temp_size
            %          strand_count=strand_count+1; % Inrease the strand count in the strand matrix
            attach_site=BIP2_site(i);  %the node number where the primer attaches
            %         new_strand=strand_count;    %new strand
            add_strand;
            Primer_anneal;
            
            %add other nuc segment of the primer
            lastnode_strand=new_strand;
            Identify_lastnode;
            Strand(lastnode+1,STRAND_NUM,new_strand) = 1 + Strand(lastnode,STRAND_NUM,new_strand);  %Add a node at the end for the new nucleotide segment
            Strand(lastnode+1,NUC_SEG,new_strand)= BIP(1,2);  %Write the nucleotide segment on the new node
            Strand(lastnode+1,COVE_BOND,new_strand)=0;
            Strand(lastnode+1,FIVE_PRIME,new_strand)=0;
            Strand(lastnode+1,THREE_PRIME,new_strand)=Strand(Strand(attach_site,COVE_BOND,new_strand),STRAND_NUM,new_strand);
            Primer_attach_extend;       %run primer attach and extend routine
        end
    end
    [t1,t2,size_struct]=size(Strand);
    current_strand=current_strand+1
end

%Identify the last node
y=look_for_node(lookfor,M)

M_count=1;
[M_r,M_c]=size(M);
while ( (M(M_count,1)~=lookfor) && (M_count < M_r) )
    M_count=M_count+1;
end
y=M_count;
clear M M_r M_c;

%Primer anneal
Strand(:,:,new_strand)=Strand(:,:,current_strand);
lastnode_strand=new_strand;
Identify_lastnode;
Strand(lastnode+1,STRAND_NUM,new_strand)=1 + max(Strand(:,STRAND_NUM,new_strand)); %Add a node at the end for the primer
Strand(lastnode+1,NUC_SEG,new_strand)= -Strand(look_for_node(attach_site,Strand(:,:,new_strand)),NUC_SEG,new_strand);  %Write the nucleotide segment on the new node
Strand(lastnode+1,COVE_BOND,new_strand)= Strand(look_for_node(attach_site,Strand(:,:,new_strand)),STRAND_NUM,new_strand);   %Write the complementary nucleotide segement attached to this node
Strand(look_for_node(attach_site,Strand(:,:,new_strand)),COVE_BOND,new_strand)=Strand(lastnode+1,STRAND_NUM,new_strand);   %Covalantly bond the primer trageted nucleotide site to the primer
Strand(lastnode+1,FIVE_PRIME,new_strand)=0;   %write the fiveprime end node
Strand(lastnode+1,THREE_PRIME,new_strand)=0;   %write the threeprime end node

%strand_length
int count=1;
int working_strandsize=0;
while (Strand(i,1,working_strand)~=0)
    i=i+1;
end
working_strandsize=i;

 %Identify the last node of the strand
[tmp_i1,tmp_i2,tmp_i3]=size(Strand(:,STRAND_NUM,lastnode_strand));
ctr1=1;
lastnode=0;

while (Strand(ctr1,1,lastnode_strand)~=0 && ctr1 < tmp_i1)
    ctr1=ctr1+1;
end

if(ctr1==tmp_i1)
    lastnode=Strand(ctr1,STRAND_NUM,lastnode_strand);
    lastnode=look_for_node(lastnode,Strand(:,:,lastnode_strand));
end

if(Strand(ctr1,1,lastnode_strand)==0)
    lastnode=Strand(ctr1-1,STRAND_NUM,lastnode_strand);
    lastnode=look_for_node(lastnode,Strand(:,:,lastnode_strand));
    %lastnode=Strand(ctr1-1,STRAND_NUM,lastnode_strand);
end

%convert strand to code
clear temp m n new_temp
temp=template;
[m,n]=size(temp);
for i=1:n
    new_temp((i-1)*4+i)=temp(i);
    if (i==1)
        new_temp((i-1)*4+1+i)=0;
        new_temp((i-1)*4+2+i)=0;
        if(n==1)
            new_temp((i-1)*4+3+i)=0;
        else
            new_temp((i-1)*4+3+i)=temp(i+1);
        end
        new_temp((i-1)*4+4+i)=10;
    elseif (i==n)
        new_temp((i-1)*4+1+i)=temp(i-1);
        new_temp((i-1)*4+2+i)=0;
        new_temp((i-1)*4+3+i)=0;
        new_temp((i-1)*4+4+i)=10;
    else
        new_temp((i-1)*4+1+i)=temp(i-1);
        new_temp((i-1)*4+2+i)=0;
        new_temp((i-1)*4+3+i)=temp(i+1);
        new_temp((i-1)*4+4+i)=10;
    end
    
end
template_c=new_temp;

    

