/* complot_ann.h */

#define COLOR_FILE_ANN "annotation.col"

int *g_annotation_base[TOTAL_SEQUENCES];
int g_annotation_max[TOTAL_SEQUENCES];
int g_annotation_max_total;
/* 0 is for total of others */

/* This file was originally copied May 5, 1998 from ssacolor.h in directory 
 *  /usr/people/zuker/development/annotate/Src on snark 
 *  from a file written by Jason Ginchereau
 * A similar  file is also used by sir_graph.c, color_table.c 
 * sir_graph_color_ann.h
 * this file is similar to color_ann.h and before sir_graph_color_ann.h 
 */

FILE *an_fp;
#include <stdio.h>
#include <math.h>

/* For non logarithmic formulas */
#define ANN_NUM_COLORS 40

float g_ann_color[ANN_NUM_COLORS][3]; 
/* 0 for red, 1 for green, 2 for blue */
int color_ann[ANN_NUM_COLORS];

/* keep ANN_NUM_COLORS<=100
 * edit the line below to change where the web page points to
 * for documentation on color annotation
 * when a html color table is generated by plt22gif or plt22ps, 
 * it points here */

const char *http_color_doc = 
"http://www.bioinfo.rpi.edu/applications/dat/form1-doc.html#ANN";

int color_table[ANN_NUM_COLORS+1] = {
  /* log probabilities: */
  0xff0000,     /* red */                /* > .999 */
  0xff1f00,
  0xff3f00,
  0xff5f00,
  0xff7f00,
  0xff9f00,
  0xffbf00,
  0xffdf00,                              /* > .99  */
  0xffff00,     /* yellow */
  0xdfff00,
  0xbfff00,
  0x9fff00,
  0x7fff00,
  0x5fff00,
  0x3fff00,                              /* > .9   */
  0x1fff00,
  0x00ff00,     /* green */
  0x00ff3f,
  0x00ff7f,
  0x00ffbf,
  0x00ffff,     /* cyan */               /* ~ .5   */
  0x00bfff,
  0x007fff,
  0x003fff,
  0x0000ff,     /* blue  */
  0x1f00ff,                              /* < .1   */
  0x3f00ff,
  0x5f00ff,
  0x7f00ff,
  0x9f00ff,
  0xbf00ff,
  0xdf00ff,     /* dark magenta */
  0xaf00cf,                              /* < .01  */
  0x7f009f,
  0x5f007f,
  0x3f005f,
  0x1f003f,
  0x09001f,
  0x040009,
  0x000000,     /* black */              /* < .001 */    
  0x000000      /* must repeat black for position ANN_NUM_COLORS*1.00 */
};

/* for background and text colors */


/***********************************************************************
 * Function getcolor() accepts 'x', a float within the range between
 * 0 and 1, and returns an integer RGB color value.
 * If 'inverse' is true, the color scheme is inversed.
 * If 'logarithmic' is true, the color value is adjusted to a
 * logarithmic scale.
 */

int get_color(double x) { 
  return((int)(x * ANN_NUM_COLORS));
}

int get_color_prob(float x) {
  x=1-x;
  if(x<.001)
    return 0;
  else if(x<.01)
    return 1;
  else if(x<.1)
    return 2; 
  else if(x<.35)
    return 3;
  else if(x<.65)
    return 4;
  else if(x<.9)
    return 5;
  else if(x<.99)
    return 6;
  else
    return 7;
}

int process_ann_file(int seq,int length) {
  char rec[120];
  int line_count;
  int base,count;
  g_annotation_max[seq]=0;
  g_annotation_base[seq][0]=0;
  for(line_count=1;line_count<=length;line_count++) {
    if(fgets(rec,119,an_fp)==NULL) {
      printf("Error!\tLine %d of annotation file does not exist.\n",
	     line_count);
      return(TRUE);
    }
    sscanf(rec,"%d%d",&base,&count);
    if(line_count!=base) {
      printf("Error!\tLine %d of annotation file does not contain %d.",
	     line_count,base);
      printf("The line is:\t%s\n",rec);
      return(TRUE);
    }
    g_annotation_base[seq][base]=count;
    if(count>g_annotation_max[seq])
      g_annotation_max[seq]=count;
  }
  return(FALSE);
}

void open_error_color_ann(void) {
  printf("Error!\tThe file %s could not be found.",COLOR_FILE_ANN);
  printf("The installation of %s was defective, since the color file, ",
	 PACKAGE_STRING);
  printf("%s, was not placed in %s.\n",COLOR_FILE_ANN,PKGDATADIR);
}

int open_ann_file(void) {
  char ann_name[120]; 
  strcpy(ann_name,COLOR_FILE_ANN);
  if((an_fp=fopen(ann_name,"r"))!=NULL) {
    printf("Using %s from current working directory\n",COLOR_FILE_ANN);
    return(FALSE);
  }
  strcpy(ann_name, PKGDATADIR);
  strcat(ann_name, COLOR_FILE_ANN);
  if ((cfp = fopen(ann_name, "rt")) != NULL) {
    printf("Found color annotation file, %s\n", ann_name);
    return(FALSE); /* COLOR_FILE_ANN was found in PKGDATADIR */
  }
  open_error_color_ann();
  printf("No annotation performed.\n");
  return(TRUE);              
}

int initialize_ann_colors(int *ann_num_colors,int prob_flag) {
  int line_count;
  int num_colors;
  char rec[120];
  float red,green,blue;
  if(open_ann_file()) 
    return(TRUE);
 if(fgets(rec,119,an_fp)==NULL) {
   printf("Error1\tLine 1 of  file %s does not exist.\n", COLOR_FILE_ANN);
   return(TRUE);
 }
 if(fgets(rec,119,an_fp)==NULL) {
   printf("Error!\tLine 2 of %s file does not exist.\n",
        COLOR_FILE_ANN);
     return(TRUE);
 }
 num_colors=0;
 sscanf(rec,"%d",&num_colors);
 if(num_colors<1) {
   printf("Error!\tLine 1 of %s is wrong: It is: %s\n",COLOR_FILE_ANN,rec);
   return(TRUE);
 }
 if(num_colors>40) {
   printf("Error!\tFirst line of %s is wrong. It is %s\n",COLOR_FILE_ANN,rec); 
   printf("No more than 40 colors are allowed.\n");
   printf("No annotation can be performed.\n");
   return(TRUE);
 }
 *ann_num_colors=num_colors;
 for(line_count=0;line_count<num_colors;line_count++) {
   if(fgets(rec,119,an_fp)==NULL) {
     printf("Error!\tLine %d of %s file does not exist.\n",
	    line_count+2,COLOR_FILE_ANN);
     printf("No annotation can be performed.\n");
     return(TRUE);
   }
   sscanf(rec,"%f%f%f",&red,&green,&blue);
   if(!prob_flag) {
     if(red<0)
       red=0.;
     if(red>.99999)
       red=.99999;
     if(green<0)
       green=0.;
     if(green>.99999)
       green=.99999; 
     if(blue<0)
       blue=0;
     if(blue>.99999)
       blue=.99999;
     g_ann_color[line_count][0]=red; 
     g_ann_color[line_count][1]=green;
     g_ann_color[line_count][2]=blue;
   }
 }
 if(!prob_flag) {
   fclose(an_fp);
   return(FALSE);
 }
 printf("Reading probability colors\n");
 /* find probability colors */
 num_colors=0;
 if(fgets(rec,119,an_fp)==NULL) {
   printf("Error!\tNumber of prob annotation colors of %s file",
	  COLOR_FILE_ANN);
   printf(" does not exist.");
   return(TRUE);
 }
 sscanf(rec,"%d",&num_colors);
 if(num_colors<1) {
   printf("Error!\tProb colors line of %s is wrong. It is: %s",
	  COLOR_FILE_ANN,rec);
   return(TRUE);
 }
 if(num_colors!=8) {
   printf("Error!\tFirst line of %s is wrong. It is %s",COLOR_FILE_ANN,rec); 
   printf("There must be 8 annotation colors.\nNo annotation can be performed.\n");
   return(TRUE);
 }
 *ann_num_colors=num_colors;
 for(line_count=0;line_count<num_colors;line_count++) {
   if(fgets(rec,119,an_fp)==NULL) {
     printf("Error!\nLine %d of probability %s file does not exist.\n",
	    line_count+2,COLOR_FILE_ANN);
     printf("No probability annotation can be performed.\n");
     return(TRUE);
   }
   sscanf(rec,"%f%f%f",&red,&green,&blue);
   if(red<0)
     red=0.;
   if(red>.99999)
     red=.99999;
   if(green<0)
     green=0.;
   if(green>.99999)
     green=.99999; 
   if(blue<0)
     blue=0;
   if(blue>.99999)
     blue=.99999;
   g_ann_color[line_count][0]=red; 
   g_ann_color[line_count][1]=green;
   g_ann_color[line_count][2]=blue;
 }
 return(FALSE);
}

int compute_ann(int *length_each,int* ann_number_colors) {
  int seq,error_occurred;
  int good_seq;
  char name[120];
  error_occurred=FALSE;
  for(seq=2;seq<=g_last_used_sequence;seq++) {
    g_annotation_base[seq]=malloc( (sizeof(int)*(length_each[seq]+1)));
    if(g_annotation_base[seq]==NULL) {
      printf("Error!\tAllocation memory for seq %d of length %d",
	     seq,length_each[seq]);
      printf("Annotation not enabled.\n");
      /* free up the ones that did work */
      for(good_seq=2;good_seq<seq;good_seq++)
	free(g_annotation_base[good_seq]);
      return (TRUE);
    }
  }
  seq=2;
  while((seq<=g_last_used_sequence)&&(!error_occurred)) { /* open file */
    strcpy(name,plot_filename_short[seq]);
    strcat(name,".ann");
    printf("Trying to open %s\n",name);
    if((an_fp=fopen(name,"r"))==NULL) {
      error_occurred=TRUE;
      printf("Error!\tCould not open %s.\n",name);
      printf("No annotation performed.\n");              
    } else {
      error_occurred=process_ann_file(seq,length_each[seq]);
      fclose(an_fp);
    }
    seq++;
  }
  if(error_occurred) {
    for(seq=2;seq<=g_last_used_sequence;seq++)
      free(g_annotation_base[seq]);
  } else {
    g_annotation_max[0]=0;
    for(seq=2;seq<=g_last_used_sequence;seq++) {
      printf("For sequence %d, the maximum p-num is %d.\n",
             seq-1,g_annotation_max[seq]);
      g_annotation_max[0]+=g_annotation_max[seq];
    }
    printf("Total pnum is %d.\n",g_annotation_max[0]);
  }
 if(!error_occurred) {
   error_occurred=initialize_ann_colors(ann_number_colors,FALSE);
 }
 return(error_occurred);
}

int compute_ann_prob(int* ann_number_colors) {
  return(initialize_ann_colors(ann_number_colors,TRUE));
}
