/* complot_setcolor.h */

#define COLOR_FILE "complot.col"

/* 0 for red, 1 for green, 2 for blue
 * color 0 is full overlap, 1 is partial,
 * colors 2 through 6 are the 5 available colors for structures
 * 8 for text, 7 for compensating base pairs
 * 9 for grid lines, 10 for labels, 11 for background  
 */
float g_color[TOTAL_COLORS][3]; 

FILE *cfp;
char color_filename[80];

/* 0-6  are black, gray, yellow ,...
 * 7 is compensating
 *for 2nd  0 is dot magnifier, 1 is miniminum size in points 
 */
float g_ps_dot_min[8][2]; 

/* Define and declare
 * #define COLOR_FILE "xxx.col"
 * FILE *cfp;
 * char color_filename[80];
 * The next three functions are from dot_col.h
 */

void read_line(char *rec) { 
  if (fgets(rec,90,cfp)==NULL) {
    printf("The %s file is too short.\n",COLOR_FILE);
    try_exit(30);
  }
}

void open_error_color(void) {
  printf("Error!\tThe file %s could not be found.",COLOR_FILE);
  printf("The installation of %s was defective, since the color file, ",
	 PACKAGE_STRING);
  printf("%s, was not placed in %s.\n",COLOR_FILE,PKGDATADIR);
  try_exit(31);
}

void open_color(void) {
  strcpy(color_filename,COLOR_FILE);
  if ( (cfp=fopen(color_filename,"r"))!=NULL ) {
    printf("The color file, %s, was found in the current working directory.\n",
	   COLOR_FILE);
    return; /* COLOR_FILE was found in pwd */
  }
  strcpy(color_filename, PKGDATADIR);
  strcat(color_filename, COLOR_FILE);
  if ((cfp = fopen(color_filename, "rt")) != NULL)
    return; /* COLOR_FILE was found in PKGDATADIR */
  open_error_color(); /* was not found in either directory */
}

void initialize_colors(void) { 
  char rec[90];
  int i;
  float size_mag,size_min;
  open_color();
  read_line(rec); /* read first line , ignore it*/
  for(i=0;i<TOTAL_COLORS;i++) { /* read and set colors */
    read_line(rec);
    sscanf(rec,"%f %f %f",
	   &g_color[i][0],&g_color[i][1],&g_color[i][2]);
  }
  read_line(rec); /* read and ignore line */
  read_line(rec);
  /* set dot magnifier and minimum size */
  for(i=0;i<=7;i++) {
    read_line(rec);
    sscanf(rec,"%f%f",&size_mag,&size_min);
    g_ps_dot_min[i][0]=size_mag;
    g_ps_dot_min[i][1]=size_min;
  }
  fclose(cfp);
}
