/* M. Zuker - January 4, 2007. This file derived from former
 * mutplot.inc 
 * The mutplot program programs reads multiple sequences from an
 * alignment file. It then computes and displays mutual information
 */ 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h> /* for isspace() */
#include <time.h>
#include <limits.h>
#include "util.h"

#ifdef HAVE_GLUTBITMAPCHARACTER
/* On the Mac, libglut does not exist separately. Search instead for
   the function glutInit. Under Linux, glut.h should be in
   /usr/include/GL and /usr/include is automatically searched.
*/
#if HAVE_LIBGLUT
# include <GL/glut.h>
#else
# include <glut.h>
#endif

#include "mutplot_input_sequence.h"
#include "mutplot_setcolor.h"

#define FALSE 0
#define TRUE  1

/* window indentifiers for glut. The order is changed from the original.
 * 1 is main, 2 is text, 3 is zoom
 */
int win_main, win_text, win_zoom; /* glutCreateWindow sets these
				     automatically */
char g_program_name[120];
int g_verbose_bp_dump_flag = FALSE; /* M. Zuker Oct. 29, 2009. */
int g_frequency_flag = FALSE;
int g_input_row = -1;
int g_input_col = -1;
int g_input_reference_sequence = -1;
int g_input_left = -1;
int g_input_right = -1;
int g_input_top = -1;
int g_input_bottom = -1;
float g_input_cutoff = -1.0;

/* left,right,top,bottom,width and height for zoom window display
 * They are in nucleotide coordinates */
int display_l = 1, display_r = 700, display_t = 1, display_b = 700;
int display_w = 700, display_h = 700;

 /* For creating a backup of a zoom window */
int saved_display_l,saved_display_r,saved_display_b,saved_display_t;

char g_post_filename[120]; /* for postscript file or label file */
char g_out_filename[120]; /* for dump file */
int  g_output_filename_defined = FALSE;

int g_i_row = 1, g_j_col = 1; /* for i.j base pair with mouse */

/* text window variables 
 * Should text display window show text window message?
 * It is used for entering a postscript filename or an MI increment 
 */
int g_message_set=FALSE; 

/* Values below for text_window_message based on g_message_set */
char *post_file_message = "Enter the output file name? (/ to abort, default: ";
char *id_file_message = "Enter the ID list file name? (/ to abort, default: ";
char *increment_message = "(/ to abort) Enter the cutoff (0.0 to 3.0) " ;
char *label_message = "What is the zoom label file name? (/ to abort) ";
char *align_message_ps =
"Enter name for alignment file (/ to abort, default: sequence_align.ps) ";
char *reference_sequence_message = "Enter name of Sequence (/ to abort)  ";
char *g_out_file_message =
"(/ to abort) Enter name for file of base pairs (Default ";
char *align_row_col_message = "Enter 2 positions of the alignment  from (1 -";
char *ref_seq_row_col_message=
"Enter 2 positions of the reference sequence from (1 -  ";
char text_window_message[100] = "            ";
char text_window_input[100]="              "; 
int data_entered=FALSE; /* for text display window */
int g_type_message; /* 1 for ps file name, 2 for MI increment & 0 for none */
FILE *file;
char g_in_align_filename[120]="        ";
char filename[120]="           ";

/* global sequence_data[i] is is the sequence from the alignment file
 * i-1, includes .'s 
 * global sequence_data_new is copied to g_sequence_data when 
 * the program restarts
 * original is a copy of what is in the input alignment file
 * previous provides a way to go back to last alignmen used 
 */

int len=1; /* same as g_sequence_length */

Si g_cutoff;
int g_count_no_offset; /* Overlapping points for main window with no offset */
double g_stack_energy[5][5][5][5]; /* for computing free energies of helices */

/* Functions for making common files.
 * given a row column and sequence it returns the length of the helix
 * within the helix from row,column to its end 
 */

void bad_arguments(void) {
  printf("Insufficient arguments:\n");
  printf("[ Usage:          mutplot alignment_file                    ]\n");
  printf("[             or  mutplot -o name_out alignment_file        ]\n");
  printf("[                                                           ]\n");
  printf("[                Valid arguments                            ]\n");
  printf("[ -f           Display base pair info as frequencies        ]\n");
  printf("[ -c cutoff    Enter cutoff 0.00<=cutoff<=3.00              ]\n"); 
  printf("[ -h           Hide mutual information window               ]\n");
  printf("[ -v           Verbose output in bp MI dump (.out)          ]\n");
  printf("[ -l row col   Select Reference Sequence position (row,col) ]\n");
  printf("[ -o name_out  Specify name of output file as               ]\n");
  printf("[              name_out.ps                                  ]\n");
  printf("[ -r j         Select sequence j as the reference sequence  ]\n");
  printf("[              Default is 1,   1<=j<=total sequences        ]\n");
  /*
    printf("[ -z left right top bottom                                ]\n");
    printf("[              Selects initial zoom window                ]\n");
*/
  try_exit(1);
}

void check_parameters(int argc,char **argv) {
  int i, input_filename_defined, space_test;
  g_input_cutoff = -1.0;
  strcpy(g_program_name,argv[0]);
  input_filename_defined = FALSE;
  g_cutoff = 500; /* really .5 */
  g_frequency_flag = FALSE;
  if(argc<2)
    bad_arguments();
  for (i=1;i<argc;i++) {
    if (strcmp(argv[i],"-o")==0) {
      if (i<(argc-1)) {
	g_output_filename_defined = TRUE;
	strcpy(g_post_filename,argv[i+1]);
	strcpy(g_out_filename,argv[i+1]);
	i++;
      }
    } else if (strcmp(argv[i],"-r")==0) {
      if(i<(argc-1)) {
	g_input_reference_sequence=atoi(argv[i+1]);
	i++;
      }
    } else if (strcmp(argv[i],"-c")==0) {
      if(i<(argc-1)) {
	sscanf(argv[i+1],"%f",&g_input_cutoff);
	if(g_input_cutoff>3.0) g_input_cutoff=3.0;
	i++;
      }
    } else if (strcmp(argv[i],"-f")==0) {
      g_frequency_flag=TRUE;
    } else if (strcmp(argv[i],"-v")==0) {
      g_verbose_bp_dump_flag = TRUE ;
    } else if (strcmp(argv[i],"-l")==0) {
      if(i<(argc-2)) {
	g_input_row=atoi(argv[i+1]);
	g_input_col=atoi(argv[i+2]);
	if(g_input_row<1) g_input_row=1;
	if(g_input_col<1) g_input_col=1;
	i+=2;
      }
    } else if(argv[i][0]=='-') {
      printf("Argument %s was ignored\n",argv[i]);
    } else {
      space_test=strcmp(argv[i]," ");
      if(space_test) {
	if(input_filename_defined)
	  printf("Argument %s was ignored\n",g_in_align_filename);
	strcpy(g_in_align_filename,argv[i]);
	input_filename_defined=TRUE;
      }
    } 
  }
  if (g_input_cutoff>=0)
    g_cutoff=(int)(1000*g_input_cutoff+.5);
  if (!input_filename_defined) {
    printf("Error!\tNo align file was specified.\n");
    bad_arguments();
  }
  if (!g_output_filename_defined) {
    strcpy(g_post_filename, g_in_align_filename);
    strcpy(g_out_filename, g_in_align_filename);
  }
  chop_suffix(g_post_filename, ".ps");
  chop_suffix(g_post_filename, ".out");
  strcat(g_post_filename,".ps");
  strcat(g_out_filename,".out");
}

FILE *popen(const char *,const char*);

void read_alignment_file(void) {
  initialize_sequence_data(g_in_align_filename, g_conserved_low_test,
			   g_conserved_min_test); 
}

void  first_initialize(int argc,char **argv) {
  check_parameters(argc,argv);
  read_alignment_file();
}

/* below are used for screen, ps and  tic marks */

int step_fun(int dis_l,int dis_r) {
  int step;
  step = (dis_r - dis_l+1)/8; /* make 8 steps  (9 tick marks total) */
  if (step < 40) {
    if (step < 3) {
      step=2;
    } else if (step > 5) {
      step = (step + 5)/10; /* make a large step a multiple of ten */
      step = step*10;
    }
  } else if (step==4) { /* make steps of 4 look like 5 */
    step = 5;
  } else if (step>90) {
    step = (step + 50)/100; /* round to nearest 100 */
    step=step*100;
  } else {
    step = (step + 25)/50; /* round to nearest 50 */
    step = step*50;
  }
  return step;
}

/* make the tick marks start at a nice place 
 * ends are treated separately and always drawn */
int start_fun(int dis_l,int step) {
  int start;
  start=dis_l+step;
  if(step>90) {
    start = (start + 60)/100; /* start at a multiple of 100 */
    start = start*100;
  }
  else if (step>=40) {
    start = (start + 32)/50; /* start at a multiple of 50 */
    start=start*50;
  } else if(step>5) {
    start = (start + 5)/10; /* start at a multiple of ten */
    start=start*10;
  }
  return start;
}
#endif
