/* mutplot_ps.c */

#include <time.h>

FILE *postfp;  /* for postscript file or label file */

void fixcolor_plot_postscript(int color) { /*  postscript colors */
  if((color>=1)&&(color<TOTAL_COLORS))
    fprintf(postfp,"\n  color_%d\n",color); 
  
}

void fixcolor_plot_postscript_define(float color_table[TOTAL_COLORS][3]) {
  /*  postscript colors */
  int color;
  fprintf(postfp,"\n%s Set colors","%%");
  fprintf(postfp,"\n%s 1 is conserved base pairs","%%");
  fprintf(postfp,"\n%s 2-6 are levels of mutual info, 2 for highest","%%");
  fprintf(postfp,"\n%s 8 is text, 9 is grid","%%");
  fprintf(postfp,"\n%s 10 is labels, 11 is background","%%");
  fprintf(postfp,"\n%s can also be set in mutplot.col\n","%%");
  for(color=1;color<TOTAL_COLORS;color++) {
    fprintf(postfp,"/color_%d {  %.3f %.3f %.3f setrgbcolor} def \n",color,
	    color_table[color][0],color_table[color][1],color_table[color][2]);
  } 
}

void draw_postscript_dots_zoom(int left,int right,int top,int bottom,
          Si *big_matrix[MAXLEN],Si cutoff,int conserved_bp_flag) {
  int row,col;
  int color;
  int previous_color;
  fixcolor_plot_postscript(1);
  previous_color=1;
  for(row=top;row<=bottom;row++) {
    for(col=left;col<=right;col++) {
      if(row<col)
        color=find_dot_color(big_matrix[row][col],cutoff,conserved_bp_flag);
      else
        color=find_dot_color(big_matrix[col][row],cutoff,conserved_bp_flag); 
      if(color>-1) {
	if(row<col) {
	  if(color!=previous_color) {
	    fixcolor_plot_postscript(color);
	    previous_color=color;
	  }
	  fprintf(postfp,"%d %d b%d\n", row-top+1,col-left+1,color);
	} else if((row!=col)&&(color<3)) {
	  fixcolor_plot_postscript(color);
	  previous_color=color;
	  fprintf(postfp,"%d %d b%d\n",  row-top+1,col-left+1,color);
	}
      }
    }
  }
}

void draw_postscript_dots(int left,int right,int top,int bottom,
          Si *big_matrix[MAXLEN],Si cutoff,int conserved_bp_flag) {
  int row,col;
  int color;
  int previous_color;
  fixcolor_plot_postscript(1);
  previous_color=1;
  for(row=top;row<=bottom;row++) {
    for(col=row+1;col<=right;col++) {
      color=find_dot_color(big_matrix[row][col],cutoff,conserved_bp_flag);
      if(color>-1) {
        if(color!=previous_color) {
	  fixcolor_plot_postscript(color);
	  previous_color=color;
	}
        fprintf(postfp,"%d %d b%d\n", row-top+1,col-left+1,color);
        if(color<3) {
	  fprintf(postfp,"%d %d b%d\n", col-top+1,row-left+1,color);
	}
      }
    }
  }
}

void single_hor_tic_post(int j_col,float scale,int grid_flag) {
  fprintf(postfp," %f 1. scale\n",scale);
  fprintf(postfp,"\n\n %f setlinewidth",0.40/scale);
  fprintf(postfp," %d 540 moveto\n",j_col+1);
  fprintf(postfp," 0 8 rlineto\n");
  if(grid_flag==TRUE) {
    fprintf(postfp,"\n stroke");
    fixcolor_plot_postscript(COLOR_GRID);
    fprintf(postfp,"\n %d 530 moveto",j_col+1);
    fprintf(postfp,"\n 0 -538 rlineto");
    fprintf(postfp,"\n stroke");
    fixcolor_plot_postscript(COLOR_TEXT);
  }
  fprintf(postfp," %d 550 moveto\n",j_col+1);
  fprintf(postfp," %f 1. scale\n",1./scale);
  fprintf(postfp,"\n -6 0 rmoveto");
  fprintf(postfp," (%d) show\n",j_col);
}

void single_hor_tic_zoom_post(int j_col,float scale,int grid_flag,int left) {
  float top_edge;
  top_edge=540.0;
  fprintf(postfp," %f 1. scale\n",scale); 
  fprintf(postfp,"\n\n %.2f setlinewidth",0.70/scale);
  fprintf(postfp," %d %.2f moveto\n",j_col-left+1,top_edge+1);
  fprintf(postfp," 0 8 rlineto\n");
  if(grid_flag==TRUE) {
    fprintf(postfp,"\n stroke");
    fixcolor_plot_postscript(COLOR_GRID);
    fprintf(postfp,"\n %d %.2f moveto",j_col-left+1,top_edge-1);
    fprintf(postfp,"\n 0 %.2f rlineto",-1*(top_edge-1));
    fprintf(postfp,"\n stroke");
    fixcolor_plot_postscript(COLOR_TEXT);
  }
  fprintf(postfp," %d %.2f moveto\n",j_col-left+1,top_edge+10);
  fprintf(postfp," %f 1. scale\n",1./scale);
  fprintf(postfp," -6 0 rmoveto\n");
  fprintf(postfp," ( %d ) show\n",j_col);
}

void plot_hor_tics_zoom_post(float scale,int grid_flag,int left,int right) { 
  int j_col,step;
  int start;
  step=step_fun(left,right);
  start=start_fun(left,step);
  single_hor_tic_zoom_post(right,scale,FALSE,left);
  single_hor_tic_zoom_post(left,scale,FALSE,left);
  for(j_col=start;j_col<(right-2*step/3); j_col=j_col+step) {
    single_hor_tic_zoom_post(j_col,scale,grid_flag,left);
  }
}

void single_ver_tic_zoom_post(int i_row,float scale,int grid_flag,int bottom) {
  float right_edge;
  right_edge=540.5;
  fprintf(postfp," 1. %f scale\n",scale);
  fprintf(postfp," %.2f %d moveto\n",right_edge,bottom-i_row+1);
  fprintf(postfp,"\n\n %f setlinewidth",0.70/scale);
  fprintf(postfp," 10 0 rlineto\n");
  if(grid_flag==TRUE) {
    fprintf(postfp,"\n stroke");
    fixcolor_plot_postscript(COLOR_GRID);
    fprintf(postfp,"\n %.2f %d moveto",right_edge-1,bottom-i_row+1);
    fprintf(postfp,"\n %.2f 0 rlineto",-1*(right_edge-1));
    fprintf(postfp,"\n stroke");
    fixcolor_plot_postscript(COLOR_TEXT);
  }
  fprintf(postfp," %.2f %d moveto\n",right_edge+10,bottom-i_row+1);
  fprintf(postfp," 1. %f scale\n",1./scale);
  fprintf(postfp," ( %d ) show\n",i_row);
}

void plot_ver_tics_zoom_post(float scale,int grid_flag,int top,int bottom) {
  int i_row,step,start;
  step=step_fun(top,bottom);
  start=start_fun(top,step);
  single_ver_tic_zoom_post(top,scale,FALSE,bottom);
  single_ver_tic_zoom_post(bottom,scale,FALSE,bottom);
  for(i_row=start;i_row<(bottom-step/2);i_row=i_row+step) {
    single_ver_tic_zoom_post(i_row,scale,grid_flag,bottom);  
  }
}

void draw_diagonal_line_post(int d_l,int d_r,int d_t,int d_b) {
  int left,top,bottom,right;
  if((d_r>=d_t)&&(d_b>=d_l)) {
    top=d_t;
    bottom=d_b;
    right=d_r;
    left=d_l;
    if(d_r>=d_b) {
      bottom=d_b;
      right=d_b;
    } else {
      right=d_r;
      bottom=d_r;
    }
    if(d_l>=d_t) {
      left=d_l;
      top=d_l;
    } else {
      top=d_t;
      left=top;
    }   
    fprintf(postfp,"\n %.2f  %.2f moveto\n", left-d_l+0.0, d_b-top+2.0);
    fprintf(postfp,"\n %.2f %.2f  lineto stroke\n",
	    right-d_l+2.0,d_b-bottom+0.0);
  }
}

void fix_dot_size_def1(int i,float hscale,float vscale,int big_dot_flag,
		       float ps_dot_min[8][2]) {
  float dot_size,new_scale;
  if(big_dot_flag==TRUE) {
    new_scale=(hscale+vscale)/2.0;
    dot_size=ps_dot_min[i][0];
    if(new_scale*dot_size<ps_dot_min[i][1])
      dot_size=ps_dot_min[i][1]/new_scale;
  } else
    dot_size=1.0;
  fprintf(postfp,"/s%d { %.3f } def\n",i,dot_size);
}

void fix_dot_size_def2(int i) {
  fprintf(postfp,"/S%d {.50 s%d mul } def\n",i,i);
  fprintf(postfp,"/NS%d {-1.0 S%d mul } def\n",i,i);
}

void fix_dot_size(int i) {
  fprintf(postfp,"/b%d {\n",i);
  fprintf(postfp," exch len exch sub 1 add \n");
  fprintf(postfp," moveto\n");
  fprintf(postfp," S%d S%d rlineto\n",i,i);  
  fprintf(postfp,"   NS%d   0 rlineto\n",i);
  fprintf(postfp,"  NS%d NS%d rlineto\n",i,i);
  fprintf(postfp,"   0   NS%d  rlineto\n",i);
  fprintf(postfp,"   S%d    0 rlineto\n",i);
  fprintf(postfp,"   S%d S%d rlineto\n",i,i);
  fprintf(postfp,"   0    S%d  rlineto\n",i);
  fprintf(postfp,"   closepath\n");
  fprintf(postfp,"   fill\n");
  fprintf(postfp,"   stroke\n");
  fprintf(postfp,"}def\n");
}

int open_ps_file(char * file_name) {    
   if ((postfp = fopen(file_name, "w")) == NULL) {
     printf("Error!\tCould not open postscript_file:  %s", file_name);
     return(1);
   }
   return (0);
}

void display_time(FILE *fp) {
  time_t now;
  now=time(NULL);
  fprintf(fp,"\n%s %s","%%",ctime(&now));
}

void general_post(char * post_filename,int left,int right,int top,int bottom, 
		  Si cutoff,float color_table[TOTAL_COLORS][3],int grid_flag,
		  int big_dot_flag,float ps_dot_min[8][2],
		  int zoom_labels_count,int zoom_labels_row[50],
		  int zoom_labels_col[50],char zoom_labels_string[50][15],
		  Si *big_matrix[MAXLEN],int post_filetype,
		  int conserved_bp_flag) {
  float hscale,vscale;
  char str[120] = "";
  int i;
  int width;
  int height;
  int error;
  float right_edge;
  float cutoff_float;
  float top_edge;  
  chop_suffix(post_filename, ".ps");
  strcat(post_filename,".ps");
  printf("Postscript file: %s\n", post_filename);
  error=open_ps_file(post_filename);
  if(error) {
    printf("Error creating postscript file #%d\n",error);
    return;
  }
  width = right - left + 1;
  height = bottom - top + 1;
  hscale = 540.0/(width + 1);
  vscale = 540.0/(height + 1);
  fprintf(postfp,"%c!PS-Adobe-3.0 EPSF-2.0 \n",'%');
  fprintf(postfp,"%sBoundingBox: 0 0 %d %d\n","%%", 612,792);
  fprintf(postfp,"%s from Postscript dotplot file \n","%%");
  fprintf(postfp,"%s Created by %s, %s\n","%%",PROGRAM_NAME, PACKAGE_STRING);
  display_time(postfp);
  fprintf(postfp,"%s\n%s\n","%%","%%");
  fprintf(postfp," %f %f scale\n",1.0,1.0);
  fprintf(postfp,"%s\n","%%");
  fprintf(postfp,"%s To change image size ....\n","%%");
  fprintf(postfp,"%s For full size image Use BoundingBox:0 0 612 792\n","%%");
  fprintf(postfp,"%s                     and 1.0 1.0 scale\n","%%");
  fprintf(postfp,"%s For image 1/4 of full size, replace 612 with 612/4\n","%%");
  fprintf(postfp,"%s        replace 792 with .25*792 and replace\n","%%");
  fprintf(postfp,"%s        1.0 with .25: Other scales are similar.\n","%%");
  fprintf(postfp,"%s        Valid scales are .05 to 1.00\n","%%");
  fprintf(postfp,"%s ONLY scale and BoundingBox lines above should be changed.\n",
	  "%%");
  fprintf(postfp,"%s\n","%%");
  fixcolor_plot_postscript_define(color_table);
  fprintf(postfp, "\n%s Dot shape and Size definitions, b1 for conserved base pairs\n", "%%");
  fprintf(postfp,"%s b2 for Mut. info of [1.4, 2.0] , b3 is  [1.0, 1.4)\n","%%");
  fprintf(postfp,"%s b4 [.70, 1.00 , b5 [.40, .70),b6 for[0.00, .40) sbox is for labels\n","%%");
  fprintf(postfp,"\n%s Change si below to adjust size of dots\n","%%");
  fprintf(postfp,"%s sl is for labels \n","%%");
  fprintf(postfp,"%s 1.0 is no overlap, s1 = 2.0 doubles size of\n","%%");
  fprintf(postfp,"%s conserved b.p dots, b1\n","%%");
  fprintf(postfp,"%s s2 = 2.0 doubles size of b2 dots [1.4, 2.0]\n","%%");
  fprintf(postfp,"%s Use %.4f to make dots 1/72 inch or 1 point\n","%%",
	  2.0/(hscale+vscale));
  fprintf(postfp,"%s Ignore Si,NSi,len \n\n","%%");
  for(i=1;i<=6;i++) {
    fix_dot_size_def1(i,hscale,vscale,big_dot_flag,ps_dot_min);
  } 
  if(zoom_labels_count>0) {
    fprintf(postfp,"/sl { .5 } def\n");
  }
  fprintf(postfp,"\n");
  for(i=1;i<=6;i++) {
    fix_dot_size_def2(i);
  }
  if(zoom_labels_count>0) {
    fprintf(postfp,"/Sl {.50 sl mul } def\n");
    fprintf(postfp,"/NSl {-1.0 Sl mul } def\n");
  }
  fprintf(postfp,"\n");
  fprintf(postfp,"/len { %d } def\n\n",height);    
  for(i=1;i<=6;i++) {
    fix_dot_size(i);
  }
  if (zoom_labels_count>0) {
    fprintf(postfp,"%s Box size for labels, neighbors \n","%%");
    fprintf(postfp,"/sbox {\n");
    fprintf(postfp,"  exch len exch sub 1 add \n");
    fprintf(postfp," moveto\n");
    fprintf(postfp,"   Sl  Sl rlineto\n");
    fprintf(postfp,"   NSl   0  rlineto\n");
    fprintf(postfp,"  NSl    NSl rlineto\n");
    fprintf(postfp,"   0   NSl  rlineto\n");
    fprintf(postfp,"   Sl    0 rlineto\n");
    fprintf(postfp,"   Sl  Sl  rlineto\n");
    fprintf(postfp,"   0    Sl  rlineto\n");
    fprintf(postfp,"   closepath\n");
    fprintf(postfp,"   fill\n");
    fprintf(postfp,"   stroke\n");
    fprintf(postfp,"}def\n\n");
  }
  /* Add some gnuplot definitions */
  fprintf(postfp,"/M {moveto} bind def\n/L {lineto} bind def\n");
  fprintf(postfp,"/R {rmoveto} bind def\n/V {rlineto} bind def\n");
  fprintf(postfp,"/N {newpath moveto} bind def\n/C {setrgbcolor} bind def\n");
  fprintf(postfp,"/f {rlineto fill} bind def\n");
 /* create background color */
  fprintf(postfp,"\n%s Set the background color  \n","%%");
  fixcolor_plot_postscript(COLOR_BACKGROUND);
  fprintf(postfp,"\n0 0 moveto");
  fprintf(postfp,"\n611 0 rlineto");
  fprintf(postfp,"\n0 791 rlineto");
  fprintf(postfp,"\n-611 0 rlineto");
  fprintf(postfp,"\n0 -791 rlineto");
  fprintf(postfp,"\nclosepath");
  fprintf(postfp,"\nfill");
  fprintf(postfp,"\n%s Set the TITLE , and other text \n","%%");
  fixcolor_plot_postscript(COLOR_TEXT);
  fprintf(postfp," /Helvetica findfont\n");
  fprintf(postfp," 16 scalefont\n");
  fprintf(postfp," setfont\n");
  fprintf(postfp," 40 750 moveto\n");
  /* M. Zuker, June 12, 2007. Add title for plot. */
  fprintf(postfp,"(MI plot for %s. %d sequences, length %d.) show\n",
	  g_in_align_filename,g_number_sequences,g_sequence_length);
  fprintf(postfp," /Helvetica findfont\n");
  fprintf(postfp," 10 scalefont\n");
  fprintf(postfp," setfont\n");
  fprintf(postfp," 50 725 moveto\n"); 
  fprintf(postfp," ( %s ) show\n",str);
  /* label colors and cutoff values */
  fprintf(postfp," 50 135 moveto\n");
  /*  fprintf(postfp," ( Name ) show\n"); */
  fprintf(postfp," 425 730 moveto\n");
  fprintf(postfp, " ( %s %.3f ) show\n","Cutoff: ",
	  ((float)cutoff)/(float)BIG_CONVERT_FACTOR);
  if(conserved_bp_flag) {
    fixcolor_plot_postscript(1);
    fprintf(postfp," 300 135 moveto\n");
    fprintf(postfp," ( Conserved Base Pairs ) show\n");
  }
  /* red */
  if(g_cutoff<=2000) {
    fixcolor_plot_postscript(2);
    fprintf(postfp," 300 120 moveto\n");
    cutoff_float=g_cutoff/(float)BIG_CONVERT_FACTOR;
    if(g_cutoff>=1400)
      fprintf(postfp," ( [%.2f, 2.00] ) show\n",cutoff_float);
    else {
      fprintf(postfp," ( [1.40, 2.00] ) show\n");
      /* green */
      fixcolor_plot_postscript(3);
      fprintf(postfp," 300 105 moveto\n");
      if(g_cutoff>=1000)
	fprintf(postfp," ( [%.2f, 2.00] ) show\n",cutoff_float);
      else {
	fprintf(postfp," ( [1.00, 1.40\\) ) show\n");
	/* blue */
	fixcolor_plot_postscript(4);
	fprintf(postfp," 300 90 moveto\n");
	if(g_cutoff>=700)
	  fprintf(postfp," ( [ %.2f, 1.00\\) ) show\n",cutoff_float);
	else {
	  fprintf(postfp," ( [0.70, 1.00\\) ) show\n");
	  /* dark yellow */
	  fixcolor_plot_postscript(5);
	  fprintf(postfp," 300 75 moveto\n");
	  if(g_cutoff>=400)
	    fprintf(postfp," ( [%.2f, 0.70\\) ) show\n",cutoff_float);
	  else {
	    fprintf(postfp," ( [0.40, 0.70\\) ) show\n");
	    fixcolor_plot_postscript(6);
	    fprintf(postfp," 300 60 moveto\n");
	    fprintf(postfp," ( [%.2f, 0.40\\) ) show\n",cutoff_float);
	  }
	}
      }
    }
  }
  fixcolor_plot_postscript(COLOR_TEXT);
  fprintf(postfp," 25 150 translate\n");
  /* label top and side ticks */ 
  fprintf(postfp," 0.70 setlinewidth\n");
  fprintf(postfp,"\n%s Make tick marks and grid lines  \n","%%");
  plot_hor_tics_zoom_post(hscale,grid_flag,left,right);
  plot_ver_tics_zoom_post(vscale,grid_flag,top,bottom);
  /* draw box or triangle */ 
  fprintf(postfp," 0.70 setlinewidth\n");
  right_edge=540.5;
  top_edge=540.5;
  /* left */
  fprintf(postfp,"%s Draw outer box  \n","%%");
  fprintf(postfp," 0 %.3f moveto 0 0 lineto stroke\n",top_edge);
  /* bottom */
  fprintf(postfp," 0 0.0 moveto %.3f 0 lineto stroke\n",right_edge);    
  /* draw top  */
  fprintf(postfp," 0 %.3f moveto %.3f %.3f lineto stroke\n",
	  top_edge,right_edge,top_edge);
  /* right */
  fprintf(postfp," %.3f 0  moveto %.3f %.3f lineto stroke\n",
	  right_edge,right_edge,top_edge);
  fprintf(postfp," %f %f scale\n",hscale,vscale);
  fprintf(postfp,"%s Draw diagonal line, then bases  \n","%%");
  fprintf(postfp,"\n %f setlinewidth",0.7/(hscale+vscale)*2.0); 
  draw_diagonal_line_post(left,right,top,bottom);
  fprintf(postfp,"%s For bases, you may adjust colors or order  \n","%%"); 
  fprintf(postfp,
          "%s Currently sequences, partial overlap, overlap  \n","%%");
  if(post_filetype==MAIN_POST)
    draw_postscript_dots(left,right,top,bottom,big_matrix,cutoff,
			 conserved_bp_flag);
  else
    draw_postscript_dots_zoom(left,right,top,bottom,big_matrix,cutoff,
			      conserved_bp_flag);
  if(zoom_labels_count>0) {
    fprintf(postfp,"%s Make Labels  \n","%%");
    fixcolor_plot_postscript(COLOR_LABEL);
    fprintf(postfp," /Helvetica-Bold findfont\n");
    fprintf(postfp," 8 scalefont\n");
    fprintf(postfp," setfont\n");
    for(i=0;i<zoom_labels_count;i++) {
      fprintf(postfp,"%d %d sbox\n", zoom_labels_row[i]-top+1,
	      zoom_labels_col[i]-left+1);
      fprintf(postfp,"\n %d %d moveto", zoom_labels_col[i]-left+2,
              bottom-zoom_labels_row[i]+1);
      fprintf(postfp," %f %f scale\n",1/hscale,1/vscale);
      fprintf(postfp," 0 -1.5 rmoveto\n");
      fprintf(postfp," (%s) show \n",zoom_labels_string[i]);
      fprintf(postfp," %f %f scale\n",hscale,vscale);
    }
  }
  fprintf(postfp,"\nshowpage\n");
  fprintf(postfp,"%sEOF\n","%%");
  fclose(postfp);
}

/* The following creates the output file for a list of base pairs based
 * on the cutoff.
 * The format is  i1 j1  i2 j2,  where (i1,j1) is (row,col) of the
 * alignment position and (i2,j2) is (row,col) of the reference sequence 
 */

FILE *outfp;
int open_out_file(char * file_name) {
  if ((outfp = fopen(file_name, "w")) == NULL) {
    printf("Error!\tCould not open out_file:  %s", file_name);
    return(1);
  }
  return (0);
}

/* October 6, 2006. Zuker adds output of mutual information. 
 *  Divide value by 1000 to get MI.
 */
void create_out_dots(Si *big_matrix[MAXLEN], Si cutoff,int conserved_bp_flag,
		     int *displayed_sequence_numbering, int length) {
  char *base_type = "-ACGU";
  char type_row_char, type_col_char;
  int k, row, col, value, sequence, type_row, type_col, fij[5][5];
  float value_float, cutoff_float, fij_f[5][5];

  cutoff_float = g_cutoff/(float)BIG_CONVERT_FACTOR;
  fprintf(outfp, "Dump of all base pairs with MI >= %.2f.\n", cutoff_float);
  if (conserved_bp_flag) 
    fprintf(outfp, "Includes conserved base pairs.\n");
  if (g_verbose_bp_dump_flag == FALSE) {
    fprintf(outfp,"Columns 1 & 2: Base pair in alignment co-ordinates\n");
    fprintf(outfp,"Columns 3 & 4: Base pair in terms of %s.\n", 
	    g_data[g_displayed_sequence].name);
    fprintf(outfp,"Columns 5: Mutual information x %d.\n\n", 
	    BIG_CONVERT_FACTOR);
  } else {
    fprintf(outfp, "Verbose output.\n\n");
  }
  for (row=1; row<=length; row++) {
    for (col=row+1; col<=length; col++) {
      value=big_matrix[row][col];
      if (value>=cutoff) {
	if ( (value<2001) || (conserved_bp_flag)) {
	  if (g_verbose_bp_dump_flag == FALSE) {
	    fprintf(outfp, "%d\t%d\t%d\t%d\t%d\n", row, col, 
		    displayed_sequence_numbering[row],
		    displayed_sequence_numbering[col],value);
	  } else {
	    value_float = (float)value/(float)BIG_CONVERT_FACTOR;
	    set_fij_int(fij,row,col);
	    set_fij(fij_f,row,col);
	    /* Make table to show entries */
	    fprintf(outfp,"\n\t\t  Table for (%d,%d).", row,col);
	    fprintf(outfp,"\tColumn: %d\tMI = %.3f\n",col,value_float);
	    fprintf(outfp,"\n\t\t| Fij |   -  |   A  |   C  |   G  |  U   | ");
	    fprintf(outfp,"Total |\n\t\t____________________________________");
	    fprintf(outfp,"______________\n");
	    for (k=0; k<=4; k++) {
	      if (k==2) 
		fprintf(outfp," Row: %4d\t|  ", row);
	      else
		fprintf(outfp,"\t\t|  ");
	      fprintf(outfp,"%c  | %.2f | %.2f | %.2f | %.2f | %.2f | %.2f  |",
		      base_type[k] , fij_f[k][0], fij_f[k][1], fij_f[k][2], 
		      fij_f[0][3], fij_f[k][4], fij_f[k][0] + fij_f[k][1] + 
		      fij_f[k][2] + fij_f[k][3] + fij_f[k][4] );
	      fprintf(outfp,"\n\t\t________________________________");
	      fprintf(outfp,"__________________\n");
	    }
	    fprintf(outfp,"\n\t\tTotal | %.2f | %.2f | %.2f | %.2f | %.2f |       |",
		    fij_f[0][0]+fij_f[1][0]+fij_f[2][0]+fij_f[3][0]+fij_f[4][0],
		    fij_f[0][1]+fij_f[1][1]+fij_f[2][1]+fij_f[3][1]+fij_f[4][1],
		    fij_f[0][2]+fij_f[1][2]+fij_f[2][2]+fij_f[3][2]+fij_f[4][2],
		    fij_f[0][3]+fij_f[1][3]+fij_f[2][3]+fij_f[3][3]+fij_f[4][3],
		    fij_f[0][4]+fij_f[1][4]+fij_f[2][4]+fij_f[3][4]+fij_f[4][4]);
	    fprintf(outfp,"\n\t\t__________________________________________________");
	    /* List of sequences for each type of pair */
	    fprintf(outfp,"\n\n\t\tId list for (%d,%d).\n", row, col);
	    for(type_row=0;type_row<=4;type_row++) {
	      for(type_col=0;type_col<=4;type_col++) {
		type_row_char=base_type[type_row];
		type_col_char=base_type[type_col];
		fprintf(outfp, "\n%c%c occurs %d times [%.1f%%]:",
			type_row_char,type_col_char,fij[type_row][type_col],
			fij_f[type_row][type_col]*100.);
		if(fij[type_row][type_col]!=0) {
		  for(sequence=1;sequence<=g_number_sequences;sequence++) {
		    if((g_data[sequence].sequence_num[row]==type_row)&&
		       (g_data[sequence].sequence_num[col]==type_col)) {
		      fprintf(outfp,"\t%s",g_data[sequence].name);
		    }
		  }
		}
	      }
	    }
	    fprintf(outfp,"\n");
	  }
	}
      }
    }
  }
}


void general_out_file(char *out_filename, Si cutoff,Si *big_matrix[MAXLEN],
		      int conserved_bp_flag,int *displayed_sequence_numbering,
		      int length) {
  int error;
  printf("Output file: %s\n", out_filename);
  error = open_out_file(out_filename);
  if(error) {
    printf("Error creating output file # %d\n",error);
    return;
  }
  create_out_dots(big_matrix,cutoff, conserved_bp_flag,
		  displayed_sequence_numbering,length);
  fclose(outfp);
}
