/* Use mutplot.col to set colors and probabilities for each color */

#define COLOR_FILE "mutplot.col"

#define TOTAL_COLORS 12
#define COLOR_TEXT 8
#define COLOR_GRID 9
#define COLOR_LABEL 10
#define COLOR_BACKGROUND 11

/* Colors:
   0 for red, 1 for green, 2 for blue
 * color 0 is full overlap, 1 is partial,
 * colors 2 through 6 are the 5 available colors for structures
 * 8 for text, 7 for compensating base pairs
 * 9 for grid lines, 10 for labels, 11 for background
 */
float g_color[TOTAL_COLORS][3]; 

float g_conserved_low_test; /* from mutual infor check for hij < this */
float g_conserved_min_test; /* and base pair occurs more often than this */

FILE *cfp;

char color_filename[80];

/* 0, not used 1 =black
 * 7 not used, 2 is red ,3 for green, 6 for yellow 
 * for 2nd  0 is dot magnifier, 1 is miniminum size in points 
 */
float g_ps_dot_min[8][2]; 

void read_line(char *rec) { 
  if (fgets(rec,90,cfp)==NULL) {
    printf("complot.col file is too short.\n");
    try_exit(43);
  }
}

void open_error_color(void) {
  printf("Error!\tThe file %s could not be found.",COLOR_FILE);
  printf("The installation of %s was defective, since the color file, ",
	 PACKAGE_STRING);
  printf("%s, was not placed in %s.\n",COLOR_FILE,PKGDATADIR);
  try_exit(44);
}

void open_color(void) {
  strcpy(color_filename,COLOR_FILE);
  if ( (cfp=fopen(color_filename,"r"))!=NULL ) {
    printf("The color file, %s, was found in the current working directory.\n",
	   COLOR_FILE);
    return; /* COLOR_FILE was found in pwd */
  }
  strcpy(color_filename, PKGDATADIR);
  strcat(color_filename, COLOR_FILE);
  if ((cfp = fopen(color_filename, "rt")) != NULL)
    return; /* COLOR_FILE was found in PKGDATADIR */
  open_error_color(); /* was not found in either directory */
}

void initialize_colors(void) { 
  char rec[90];
  int i;
  float size_mag,size_min;
  open_color();
  read_line(rec); /* read first line , ignore it*/
  for(i=0;i<TOTAL_COLORS;i++) { /* read and set colors */
    read_line(rec);
    sscanf(rec,"%f %f %f",
	   &g_color[i][0],&g_color[i][1],&g_color[i][2]);
  }
  read_line(rec); /* read and ignore line */
  read_line(rec);
  /* set dot magnifier and minimum size */
  for(i=0;i<=7;i++) {
    read_line(rec);
    sscanf(rec,"%f%f",&size_mag,&size_min);
    g_ps_dot_min[i][0]=size_mag;
    g_ps_dot_min[i][1]=size_min;
  }
  read_line(rec); /* read and ignore line */
  read_line(rec);  
  read_line(rec); 
  sscanf(rec,"%f%f",&g_conserved_low_test,&g_conserved_min_test);
  fclose(cfp);
}

int find_dot_color(Si value,Si cutoff,int conserved_bp_flag) {
  if (value<cutoff) return -1;
  if (value<400) return 6;
  if (value<700) return 5;
  if (value <1000) return 4;
  if (value <1400) return 3;
  if (value < 2001) return 2;
  if (conserved_bp_flag) return 1;
  else return -1; 
}
